// ----------------------------------------------------------------------------
//
// TLV5616.h
// Runs on TM4C123
// Use SSI1 to send a 16-bit code to the TLV5616.
// Spring 2025
//
// ----------------------------------------------------------------------------


// SSIClk (SCLK)  connected to PD0
// SSIFss (FS)    connected to PD1
// SSITx  (DIN)   connected to PD3
#ifndef __TLV5616_H__
#define __TLV5616_H__
#include <stdint.h>

// --------------     DAC_Init     --------------------------------------------
//
// Initialize TLV5616 12-bit DAC
// assumes bus clock is 80 MHz
// inputs: initial voltage output (0 to 4095)
// outputs:none
//
void DAC_Init(uint16_t data);

// --------------      DAC_Out     --------------------------------------------
//
// Send data to TLV5616 12-bit DAC
// inputs:  voltage output (0 to 4095)
// outputs: none
//
void DAC_Out(uint16_t code);

// -----------------   DAC_OutNonBlocking      --------------------------------
//
// Send data to TLV5616 12-bit DAC without
// waiting for a response.  This is useful
// in audio applications where DAC outputs
// are relatively infrequent and where the
// DAC response is meaningless.  Since the
// state of the FIFO is not checked before
// sending, data may be lost.
// inputs:  voltage output (0 to 4095)
// outputs: none
//
#define DAC_OutNonBlocking(code) SSI1_DR_R = code;     

#endif
